@@ -18,6 +18,8 @@
 #include "pqxx/compiler.h"
 
 #include <algorithm>
+#include <cassert>
+#include <cerrno>
 #include <cstdio>
 #include <ctime>
 #include <stdexcept>
@@ -26,9 +28,11 @@
 #include <sys/select.h>
 #else
 #include <sys/types.h>
-#ifdef HAVE_UNISTD_H
+#if defined(HAVE_UNISTD_H)
 #include <unistd.h>
-#endif	// HAVE_UNISTD_H
+#elif defined(_WIN32)
+#include <winsock2.h>
+#endif
 #endif	// PQXX_HAVE_SYS_SELECT_H
 
 #include "libpq-fe.h"
@@ -52,6 +56,29 @@ static void pqxxNoticeCaller(void *arg, const char *Msg)
 {
   if (arg && Msg) (*static_cast<pqxx::noticer *>(arg))(Msg);
 }
+
+
+#ifdef PQXX_SELECT_ACCEPTS_NULL
+  // The always-empty fd_set
+static fd_set *const fdset_none = 0;
+#else	// PQXX_SELECT_ACCEPTS_NULL
+static fd_set emptyfd;	// Relies on zero-initialization
+static fd_set *const fdset_none = &emptyfd;
+#endif	// PQXX_SELECT_ACCEPTS_NULL
+
+
+
+// Concentrate stupid "old-style cast" warnings for GNU libc in one place, and
+// by using "C" linkage, perhaps silence them altogether.
+static void set_fdbit(int f, fd_set *s)
+{
+  FD_SET(f, s);
+}
+
+static void clear_fdmask(fd_set *mask)
+{
+  FD_ZERO(mask);
+}
 }
 
 
@@ -61,9 +88,10 @@ pqxx::connection_base::connection_base(const string &ConnInfo) :
   m_Trans(),
   m_Noticer(),
   m_Trace(0),
-  m_fdmask()
+  m_caps(),
+  m_inhibit_reactivation(false),
+  m_reactivation_avoidance(0)
 {
-  clear_fdmask();
 }
 
 
@@ -73,9 +101,10 @@ pqxx::connection_base::connection_base(const char ConnInfo[]) :
   m_Trans(),
   m_Noticer(),
   m_Trace(0),
-  m_fdmask()
+  m_caps(),
+  m_inhibit_reactivation(false),
+  m_reactivation_avoidance(0)
 {
-  clear_fdmask();
 }
 
 
@@ -85,10 +114,24 @@ int pqxx::connection_base::backendpid() const throw ()
 }
 
 
+int pqxx::connection_base::sock() const throw ()
+{
+  return m_Conn ? PQsocket(m_Conn) : -1;
+}
+
+
 void pqxx::connection_base::activate()
 {
   if (!is_open())
   {
+    if (m_inhibit_reactivation)
+      throw broken_connection("Could not reactivate connection; "
+	  "reactivation is inhibited");
+
+    // If any objects were open that didn't survive the closing of our
+    // connection, don't try to reactivate
+    if (m_reactivation_avoidance) return;
+
     startconnect();
     completeconnect();
 
@@ -112,6 +155,14 @@ void pqxx::connection_base::deactivate()
       throw logic_error("Attempt to deactivate connection while " +
 	  		m_Trans.get()->description() + " still open");
   }
+
+  if (m_reactivation_avoidance)
+  {
+    process_notice("Attempt to deactivate connection while it is in a state "
+	"that cannot be fully recovered later (ignoring)");
+    return;
+  }
+
   dropconnect();
   disconnect();
 }
@@ -157,7 +208,7 @@ string pqxx::connection_base::RawGetVar(const string &Var)
 void pqxx::connection_base::SetupState()
 {
   if (!m_Conn)
-    throw logic_error("libpqxx internal error: SetupState() on no connection");
+    throw internal_error("SetupState() on no connection");
 
   if (Status() != CONNECTION_OK)
   {
@@ -167,6 +218,8 @@ void pqxx::connection_base::SetupState()
     throw runtime_error(Msg);
   }
 
+  read_capabilities();
+
   const PSMap::const_iterator prepared_end(m_prepared.end());
   for (PSMap::iterator p = m_prepared.begin(); p != prepared_end; ++p)
     p->second.registered = false;
@@ -187,9 +240,8 @@ void pqxx::connection_base::SetupState()
     if (!m_Trans.get())
     {
       // Jump through a few hoops to satisfy dear old g++ 2.95
-      auto_ptr<nontransaction>
-	nap(new nontransaction(*this, "connection_setup"));
-      n = nap;
+      auto_ptr<nontransaction> p(new nontransaction(*this,"connection_setup"));
+      n = p;
     }
     pipeline p(*m_Trans.get(), "restore_state");
     p.retain(m_Triggers.size() + m_Vars.size());
@@ -205,7 +257,7 @@ void pqxx::connection_base::SetupState()
         // issue just one LISTEN for each event.
         if (i->first != Last)
         {
-	  p.insert("LISTEN \"" + i->first + "\"");
+          p.insert("LISTEN \"" + i->first + "\"");
           Last = i->first;
         }
       }
@@ -221,12 +273,98 @@ void pqxx::connection_base::SetupState()
 }
 
 
+void pqxx::connection_base::check_result(const result &R, const char Query[])
+{
+  if (!is_open()) throw broken_connection();
+
+  // A shame we can't detect out-of-memory to turn this into a bad_alloc...
+  if (!R) throw runtime_error(ErrMsg());
+
+  try
+  {
+    R.CheckStatus(Query);
+  }
+  catch (const exception &e)
+  {
+    /* If the connection is broken, we'd expect is_open() to return false, since
+     * PQstatus() is supposed to return CONNECTION_BAD.
+     *
+     * It turns out that, at least for the libpq in PostgreSQL 8.0 and the 8.1
+     * prerelease I'm looking at now (we're writing July 2005), libpq will only
+     * abandon CONNECTION_OK if the errno code for the broken connection is
+     * exactly EPIPE or ECONNRESET.  This is usually fine for local connections,
+     * but ignores the wide range of fatal errors that can occur on TCP/IP
+     * sockets, such as extreme out-of-memory conditions, hardware failures,
+     * severed network connections, and serious software errors.  In these cases
+     * libpq will return an error result that is (as far as I can make out)
+     * indistinguishable from a server-side error, but will retain its state
+     * indication of CONNECTION_OK--thus inviting the possibly false impression
+     * that the query failed to execute (in reality it may have executed
+     * successfully) and the definitely false impression that the connection is
+     * still in a workable state.
+     *
+     * Tom Lane was reluctant to change this behaviour, saying that it lacks
+     * robustness to give up on the connection "at the first hint of trouble"
+     * when the socket may conceivably still be usable.  I have suggested that
+     * apart from EINTR, EAGAIN and EWOULDBLOCK, which are already handled as
+     * special cases, an error result from a socket probably indicates that the
+     * operating system has decided the error was not likely to be recoverable,
+     * and second-guessing it is not likely to be productive; and that the
+     * choice to abandon a connection or attempt to revive it is probably best
+     * made before a result for the ongoing query is returned.
+     *
+     * A particular worry is connection timeout.  One user observed a 15-minute
+     * period of inactivity when he pulled out a network cable, after which
+     * libpq finally returned a result object containing an error (the error
+     * message for connection failure is subject to translation, by the way, and
+     * is also subject to a bug where random data may currently be embedded in
+     * it in place of the system's explanation of the error, so even attempting
+     * to recognize the string is not a reliable workaround) but said the
+     * connection was still operational.
+     */
+    if (!consume_input()) throw broken_connection(e.what());
+    const int fd = sock();
+    if (fd < 0) throw broken_connection(e.what());
+    fd_set errs;
+    clear_fdmask(&errs);
+    int sel;
+    do
+    {
+      timeval nowait = { 0, 0 };
+      set_fdbit(fd, &errs);
+      sel = select(fd+1, 0, 0, &errs, &nowait);
+    } while (sel == -1 && errno == EINTR);
+
+    switch (sel)
+    {
+    case -1:
+      switch (errno)
+      {
+      case EBADF:
+      case EINVAL:
+        throw broken_connection(e.what());
+      case ENOMEM:
+        throw bad_alloc();
+      }
+    case 0:
+      break;
+    case 1:
+      throw broken_connection(e.what());
+    }
+
+    throw;
+  }
+}
+
+
 void pqxx::connection_base::disconnect() throw ()
 {
   if (m_Conn)
   {
     PQfinish(m_Conn);
     m_Conn = 0;
+    // When we activate again, the server may be different!
+    for (int i=0; i<cap_end; ++i) m_caps[i] = false;
   }
 }
 
@@ -277,34 +415,34 @@ void pqxx::connection_base::process_notice(const char msg[]) throw ()
     {
       if (msg[len-1] == '\n')
       {
-	process_notice_raw(msg);
+        process_notice_raw(msg);
       }
       else try
       {
-	// Newline is missing.  Try the C++ string version of this function.
-	process_notice(string(msg));
+        // Newline is missing.  Try the C++ string version of this function.
+        process_notice(string(msg));
       }
       catch (const exception &)
       {
-	// If we can't even do that, use plain old buffer copying instead
-	// (unavoidably, this will break up overly long messages!)
-	const char separator[] = "[...]\n";
-	char buf[1007];
-	size_t bytes = sizeof(buf)-sizeof(separator)-1;
-	size_t written;
-	strcpy(&buf[bytes], separator);
-	// Write all chunks but last.  Each will fill the buffer exactly.
-	for (written = 0; (written+bytes) < len; written += bytes)
-	{
-	  memcpy(buf, &msg[written], bytes);
-	  process_notice_raw(buf);
-	}
-	// Write any remaining bytes (which won't fill an entire buffer)
-	bytes = len-written;
-	memcpy(buf, &msg[written], bytes);
-	// Add trailing nul byte, plus newline unless there already is one
-	strcpy(&buf[bytes], &"\n"[buf[bytes-1]=='\n']);
-	process_notice_raw(buf);
+        // If we can't even do that, use plain old buffer copying instead
+        // (unavoidably, this will break up overly long messages!)
+        const char separator[] = "[...]\n";
+        char buf[1007];
+        size_t bytes = sizeof(buf)-sizeof(separator)-1;
+        size_t written;
+        strcpy(&buf[bytes], separator);
+        // Write all chunks but last.  Each will fill the buffer exactly.
+        for (written = 0; (written+bytes) < len; written += bytes)
+        {
+          memcpy(buf, &msg[written], bytes);
+          process_notice_raw(buf);
+        }
+        // Write any remaining bytes (which won't fill an entire buffer)
+        bytes = len-written;
+        memcpy(buf, &msg[written], bytes);
+        // Add trailing nul byte, plus newline unless there already is one
+        strcpy(&buf[bytes], &"\n"[buf[bytes-1]=='\n']);
+        process_notice_raw(buf);
       }
     }
   }
@@ -353,7 +491,13 @@ void pqxx::connection_base::AddTrigger(pqxx::trigger *T)
     const string LQ("LISTEN \"" + T->name() + "\"");
     result R( PQexec(m_Conn, LQ.c_str()) );
 
-    if (is_open()) try {R.CheckStatus(LQ);} catch (const broken_connection &) {}
+    if (is_open()) try
+    {
+      check_result(R,LQ.c_str());
+    }
+    catch (const broken_connection &)
+    {
+    }
     m_Triggers.insert(NewVal);
   }
   else
@@ -392,7 +536,7 @@ void pqxx::connection_base::RemoveTrigger(pqxx::trigger *T) throw ()
       m_Triggers.erase(i);
 
       if (m_Conn && (R.second == ++R.first))
-	Exec(("UNLISTEN \"" + T->name() + "\"").c_str(), 0);
+        Exec(("UNLISTEN \"" + T->name() + "\"").c_str(), 0);
     }
   }
   catch (const exception &e)
@@ -402,15 +546,15 @@ void pqxx::connection_base::RemoveTrigger(pqxx::trigger *T) throw ()
 }
 
 
-void pqxx::connection_base::consume_input() throw ()
+bool pqxx::connection_base::consume_input() throw ()
 {
-  PQconsumeInput(m_Conn);
+  return PQconsumeInput(m_Conn) != 0;
 }
 
 
 bool pqxx::connection_base::is_busy() const throw ()
 {
-  return PQisBusy(m_Conn);
+  return PQisBusy(m_Conn) != 0;
 }
 
 
@@ -440,7 +584,7 @@ int pqxx::connection_base::get_notifs()
     {
       try
       {
-	process_notice("Exception in trigger handler '" +
+        process_notice("Exception in trigger handler '" +
 		       i->first +
 		       "': " +
 		       e.what() +
@@ -448,13 +592,13 @@ int pqxx::connection_base::get_notifs()
       }
       catch (const bad_alloc &)
       {
-	// Out of memory.  Try to get the message out in a more robust way.
-	process_notice("Exception in trigger handler, "
+        // Out of memory.  Try to get the message out in a more robust way.
+        process_notice("Exception in trigger handler, "
 	    "and also ran out of memory\n");
       }
       catch (const exception &)
       {
-	process_notice("Exception in trigger handler "
+        process_notice("Exception in trigger handler "
 	    "(compounded by other error)\n");
       }
     }
@@ -512,10 +656,8 @@ pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
     if (is_open()) R = PQexec(m_Conn, Query);
   }
 
-  if (!is_open()) throw broken_connection();
-  // A shame we can't detect out-of-memory to turn this into a bad_alloc...
-  if (!R) throw runtime_error(ErrMsg());
-  R.CheckStatus(Query);
+  check_result(R, Query);
+
   get_notifs();
   return R;
 }
@@ -575,9 +717,8 @@ pqxx::result pqxx::connection_base::pq_exec_prepared(const string &pname,
 #ifdef PQXX_HAVE_PQEXECPREPARED
   result R(PQexecPrepared(m_Conn, pname.c_str(), nparams, params, 0, 0, 0));
 
-  if (!is_open()) throw broken_connection();
-  if (!R) throw runtime_error(ErrMsg());
-  R.CheckStatus(pname);
+  check_result(R, pname.c_str());
+
   get_notifs();
   return R;
 #else
@@ -593,7 +734,10 @@ pqxx::result pqxx::connection_base::pq_exec_prepared(const string &pname,
 
 void pqxx::connection_base::Reset()
 {
-  clear_fdmask();
+  if (m_inhibit_reactivation)
+    throw broken_connection("Could not reset connection: "
+	"reactivation is inhibited");
+  if (m_reactivation_avoidance) return;
 
   // Forget about any previously ongoing connection attempts
   dropconnect();
@@ -603,7 +747,6 @@ void pqxx::connection_base::Reset()
     // Reset existing connection
     PQreset(m_Conn);
     SetupState();
-    clear_fdmask();
   }
   else
   {
@@ -616,9 +759,11 @@ void pqxx::connection_base::Reset()
 void pqxx::connection_base::close() throw ()
 {
 #ifdef PQXX_QUIET_DESTRUCTORS
-  set_noticer(PGSTD::auto_ptr<noticer>(new nonnoticer()));
+  auto_ptr<noticer> n(new nonnoticer());
+  set_noticer(n);
 #endif
-  clear_fdmask();
+  inhibit_reactivation(false);
+  m_reactivation_avoidance = 0;
   try
   {
     if (m_Trans.get())
@@ -636,7 +781,6 @@ void pqxx::connection_base::close() throw ()
   catch (...)
   {
   }
-  clear_fdmask();
 }
 
 
@@ -692,7 +836,7 @@ void pqxx::connection_base::UnregisterTransaction(transaction_base *T)
 void pqxx::connection_base::MakeEmpty(pqxx::result &R)
 {
   if (!m_Conn)
-    throw logic_error("libpqxx internal error: MakeEmpty() on null connection");
+    throw internal_error("MakeEmpty() on null connection");
 
   R = result(PQmakeEmptyPGresult(m_Conn, PGRES_EMPTY_QUERY));
 }
@@ -709,8 +853,7 @@ const string theWriteTerminator = "\\.";
 bool pqxx::connection_base::ReadCopyLine(string &Line)
 {
   if (!is_open())
-    throw logic_error("libpqxx internal error: "
-	              "ReadCopyLine() without connection");
+    throw internal_error("ReadCopyLine() without connection");
 
   Line.erase();
   bool Result;
@@ -724,16 +867,12 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
 
     case -1:
       for (result R(PQgetResult(m_Conn)); R; R=PQgetResult(m_Conn))
-      {
-	if (!is_open()) throw broken_connection();
-	R.CheckStatus("[END COPY]");
-      }
+	check_result(R, "[END COPY]");
       Result = false;
       break;
 
     case 0:
-      throw logic_error("libpqxx internal error: "
-	  "table read inexplicably went asynchronous");
+      throw internal_error("table read inexplicably went asynchronous");
 
     default:
       if (Buf)
@@ -790,8 +929,7 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
 void pqxx::connection_base::WriteCopyLine(const string &Line)
 {
   if (!is_open())
-    throw logic_error("libpqxx internal error: "
-	              "WriteCopyLine() without connection");
+    throw internal_error("WriteCopyLine() without connection");
 
   const string L = Line + '\n';
   const char *const LC = L.c_str();
@@ -820,20 +958,18 @@ void pqxx::connection_base::EndCopyWrite()
   case -1:
     throw runtime_error("Write to table failed: " + string(ErrMsg()));
   case 0:
-    throw logic_error("libpqxx internal error: "
-	    "table write is inexplicably asynchronous");
+    throw internal_error("table write is inexplicably asynchronous");
   case 1:
     // Normal termination.  Retrieve result object.
     break;
 
   default:
-    throw logic_error("libpqxx internal error: "
-	    "unexpected result " + to_string(Res) + " from PQputCopyEnd()");
+    throw internal_error("unexpected result " + to_string(Res) + " "
+	"from PQputCopyEnd()");
   }
 
   const result R(PQgetResult(m_Conn));
-  if (!is_open()) throw broken_connection();
-  R.CheckStatus("[END COPY]");
+  check_result(R, "[END COPY]");
 
 #else
   WriteCopyLine(theWriteTerminator);
@@ -863,50 +999,33 @@ pqxx::internal::pq::PGresult *pqxx::connection_base::get_result()
 
 namespace
 {
-#ifdef PQXX_SELECT_ACCEPTS_NULL
-  // The always-empty fd_set
-  fd_set *const fdset_none = 0;
-#else	// PQXX_SELECT_ACCEPTS_NULL
-  fd_set emptyfd;	// Relies on zero-initialization
-  fd_set *const fdset_none = &emptyfd;
-#endif	// PQXX_SELECT_ACCEPTS_NULL
+void wait_fd(int fd, bool forwrite=false, timeval *tv=0)
+{
+  if (fd < 0) throw pqxx::broken_connection();
+  fd_set s;
+  clear_fdmask(&s);
+  set_fdbit(fd, &s);
+  select(fd+1, (forwrite?fdset_none:&s), (forwrite?&s:fdset_none), &s, tv);
+}
 } // namespace
 
 
-int pqxx::connection_base::set_fdmask() const
-{
-  if (!m_Conn) throw broken_connection();
-  const int fd = PQsocket(m_Conn);
-  if (fd < 0) throw broken_connection();
-  FD_SET(fd, &m_fdmask);
-  return fd;
-}
-
-
-void pqxx::connection_base::clear_fdmask() throw ()
-{
-  FD_ZERO(&m_fdmask);
-}
-
 void pqxx::connection_base::wait_read() const
 {
-  const int fd = set_fdmask();
-  select(fd+1, &m_fdmask, fdset_none, &m_fdmask, 0);
+  wait_fd(sock());
 }
 
 
 void pqxx::connection_base::wait_read(long seconds, long microseconds) const
 {
   timeval tv = { seconds, microseconds };
-  const int fd = set_fdmask();
-  select(fd+1, &m_fdmask, fdset_none, &m_fdmask, &tv);
+  wait_fd(sock(), false, &tv);
 }
 
 
 void pqxx::connection_base::wait_write() const
 {
-  const int fd = set_fdmask();
-  select(fd+1, fdset_none, &m_fdmask, &m_fdmask, 0);
+  wait_fd(sock(), true);
 }
 
 int pqxx::connection_base::await_notification()
@@ -935,3 +1054,35 @@ int pqxx::connection_base::await_notification(long seconds, long microseconds)
 }
 
 
+void pqxx::connection_base::read_capabilities() throw ()
+{
+  int v = 0;
+#ifdef PQXX_HAVE_PQSERVERVERSION
+  if (m_Conn) v = PQserverVersion(m_Conn);
+#endif
+
+  m_caps[cap_create_table_with_oids] = (v >= 80000);
+  m_caps[cap_cursor_scroll] = (v >= 70400);
+
+  // TODO: Find out which backend versions support these
+  //m_caps[cap_cursor_with_hold] = ();
+  //m_caps[cap_cursor_update] = ();
+}
+
+
+// TODO: Inline this, once assertions are known to be good
+void pqxx::connection_base::reactivation_avoidance_add(int n) throw ()
+{
+  if (is_open()) m_reactivation_avoidance += n;
+  if (m_reactivation_avoidance < 0)
+    throw internal_error("Negative reactivation avoidance on connection");
+}
+
+// TODO: Inline this, once assertions are known to be good
+void pqxx::connection_base::reactivation_avoidance_dec() throw ()
+{
+  if (m_reactivation_avoidance <= 0)
+    throw internal_error("Reactivation avoidance falling below zero");
+  --m_reactivation_avoidance;
+}
+
