@@ -26,14 +26,32 @@
 using namespace PGSTD;
 
 
-unsigned char pqxx::cursor_base::s_dummy;
+namespace
+{
+/// Compute actual displacement based on requested and reported displacements
+pqxx::cursor_base::difference_type adjust(
+    pqxx::cursor_base::difference_type d, 
+    pqxx::cursor_base::difference_type r)
+{
+  const pqxx::cursor_base::difference_type hoped = labs(d);
+  pqxx::cursor_base::difference_type actual = r;
+  if (hoped < 0 || r < hoped) ++actual;
+  return (d < 0) ? -actual : actual;
+}
+}
+
+
 
 pqxx::cursor_base::cursor_base(transaction_base *context,
     const string &cname,
     bool embellish_name) :
   m_context(context),
   m_done(false),
-  m_name(cname)
+  m_name(cname),
+  m_adopted(false),
+  m_ownership(loose),
+  m_lastfetch(),
+  m_lastmove()
 {
   if (embellish_name)
   {
@@ -45,13 +63,13 @@ pqxx::cursor_base::cursor_base(transaction_base *context,
 
 int pqxx::cursor_base::get_unique_cursor_num()
 {
-  if (!m_context) throw logic_error("libpqxx internal error: "
-      "cursor in get_unique_cursor_num() has no transaction");
+  if (!m_context) throw internal_error("cursor in get_unique_cursor_num() "
+      "has no transaction");
   return m_context->GetUniqueCursorNum();
 }
 
 
-string pqxx::cursor_base::stridestring(pqxx::cursor_base::difference_type n)
+string pqxx::cursor_base::stridestring(difference_type n)
 {
   /* Some special-casing for ALL and BACKWARD ALL here.  We used to use numeric
    * "infinities" for difference_type for this (the highest and lowest possible
@@ -67,25 +85,177 @@ string pqxx::cursor_base::stridestring(pqxx::cursor_base::difference_type n)
 }
 
 
+void pqxx::cursor_base::declare(const string &query,
+    accesspolicy ap,
+    updatepolicy up,
+    ownershippolicy op,
+    bool hold)
+{
+  stringstream cq, qn;
+
+  cq << "DECLARE \"" << name() << "\" ";
+
+  if (m_context->conn().supports(connection_base::cap_cursor_scroll))
+  {
+    if (ap == forward_only) cq << "NO ";
+    cq << "SCROLL ";
+  }
+
+  cq << "CURSOR ";
+
+  if (hold)
+  {
+    if (!m_context->conn().supports(connection_base::cap_cursor_with_hold))
+      throw runtime_error("Cursor " + name() + " "
+	  "created for use outside of its originating transaction, "
+	  "but this backend version does not support that.");
+    cq << "WITH HOLD ";
+  }
+
+  cq << "FOR " << query << ' ';
+
+  if (up != update) cq << "FOR READ ONLY ";
+  else if (!m_context->conn().supports(connection_base::cap_cursor_update))
+    throw runtime_error("Cursor " + name() + " "
+	"created as updatable, "
+	"but this backend version does not support that.");
+  else cq << "FOR UPDATE ";
+
+  qn << "[DECLARE " << name() << ']';
+  m_context->exec(cq, qn.str());
+
+  // If we're creating a WITH HOLD cursor, noone is going to destroy it until
+  // after this transaction.  That means the connection cannot be deactivated
+  // without losing the cursor.
+  m_ownership = op;
+  if (op==loose) m_context->reactivation_avoidance_inc();
+}
+
+
+void pqxx::cursor_base::adopt(ownershippolicy op)
+{
+  // If we take responsibility for destroying the cursor, that's one less reason
+  // not to allow the connection to be deactivated and reactivated.
+  if (op==owned) m_context->reactivation_avoidance_dec();
+  m_adopted = true;
+  m_ownership = op;
+}
+
+
+void pqxx::cursor_base::close() throw ()
+{
+  if (m_ownership==owned)
+  {
+    try { m_context->exec("CLOSE " + name()); } catch (const exception &) { }
+
+    if (m_adopted) m_context->reactivation_avoidance_dec();
+    m_ownership = loose;
+  }
+}
+
+
+pqxx::result pqxx::cursor_base::fetch(difference_type n)
+{
+  result r;
+  if (n) 
+  {
+    // We cache the last-executed fetch query.  If the current fetch uses the
+    // same distance, we just re-use the cached string instead of composing a
+    // new one.
+    const string fq(
+	(n == m_lastfetch.dist) ?
+         m_lastfetch.query :
+         "FETCH " + stridestring(n) + " IN \"" + name() + "\"");
+
+    // Set m_done on exception (but no need for try/catch)
+    m_done = true;
+    r = m_context->exec(fq);
+    if (!r.empty()) m_done = false;
+  }
+  return r;
+}
+
+
+pqxx::result pqxx::cursor_base::fetch(difference_type n, difference_type &d)
+{
+  result r(fetch(n));
+  d = adjust(n, r.size());
+  return r;
+}
+
+
+template<> void
+pqxx::cursor_base::check_displacement<pqxx::cursor_base::forward_only>(
+    difference_type d)
+{
+  if (d < 0)
+    throw logic_error("Attempt to move cursor " + name() + " "
+	"backwards (this cursor is only allowed to move forwards)");
+}
+
+
+pqxx::cursor_base::difference_type pqxx::cursor_base::move(difference_type n)
+{
+  if (!n) return 0;
+
+  const string mq(
+      (n == m_lastmove.dist) ?
+      m_lastmove.query :
+      "MOVE " + stridestring(n) + " IN \"" + name() + "\"");
+
+  // Set m_done on exception (but no need for try/catch)
+  m_done = true;
+  const result r(m_context->exec(mq));
+
+  // Starting with the libpq in PostgreSQL 7.4, PQcmdTuples() (which we call
+  // indirectly here) also returns the number of rows skipped by a MOVE
+  difference_type d = r.affected_rows();
+
+  // We may not have PQcmdTuples(), or this may be a libpq version that doesn't
+  // implement it for MOVE yet.  We'll also get zero if we decide to use a
+  // prepared statement for the MOVE.
+  if (!d)
+  {
+    static const string StdResponse("MOVE ");
+    if (strncmp(r.CmdStatus(), StdResponse.c_str(), StdResponse.size()) != 0)
+      throw internal_error("cursor MOVE returned "
+	  "'" + string(r.CmdStatus()) + "' "
+	  "(expected '" + StdResponse + "')");
+
+    from_string(r.CmdStatus()+StdResponse.size(), d);
+  }
+  m_done = (d != n);
+  return d;
+}
+
+
+pqxx::cursor_base::difference_type
+pqxx::cursor_base::move(difference_type n, difference_type &d)
+{
+  const difference_type got = move(n);
+  d = adjust(n, got);
+  return got;
+}
+
+
 pqxx::icursorstream::icursorstream(pqxx::transaction_base &context,
     const string &query,
     const string &basename,
     difference_type Stride) :
-  cursor_base(&context, basename),
+  super(&context, query, basename),
   m_stride(Stride),
   m_realpos(0),
   m_reqpos(0),
   m_iterators(0)
 {
   set_stride(Stride);
-  declare(query);
 }
 
 
 pqxx::icursorstream::icursorstream(transaction_base &Context,
     const result::field &Name,
     difference_type Stride) :
-  cursor_base(&Context, Name.c_str(), false),
+  super(&Context, Name.c_str()),
   m_stride(Stride),
   m_realpos(0),
   m_reqpos(0),
@@ -102,21 +272,9 @@ void pqxx::icursorstream::set_stride(difference_type n)
   m_stride = n;
 }
 
-void pqxx::icursorstream::declare(const string &query)
+pqxx::result pqxx::icursorstream::fetchblock()
 {
-  // TODO: Add NO SCROLL if backend supports it (7.4 or better)
-  stringstream cq, qn;
-  cq << "DECLARE \"" << name() << "\" CURSOR FOR " << query << " FOR READ ONLY";
-  qn << "[DECLARE " << name() << ']';
-  m_context->exec(cq, qn.str());
-}
-
-
-pqxx::result pqxx::icursorstream::fetch()
-{
-  result r(m_context->exec("FETCH " + stridestring(m_stride) + " "
-	"IN \""+name()+"\""));
-  if (r.empty()) m_done = true;
+  const result r(fetch(m_stride));
   m_realpos += r.size();
   return r;
 }
@@ -124,9 +282,7 @@ pqxx::result pqxx::icursorstream::fetch()
 
 pqxx::icursorstream &pqxx::icursorstream::ignore(streamsize n)
 {
-  m_context->exec("MOVE " + stridestring(n) + " IN \"" + name() + "\"");
-  // TODO: Try to get actual number of moved tuples!
-  m_realpos += n;
+  m_realpos += move(n);
   return *this;
 }
 
@@ -177,7 +333,7 @@ void pqxx::icursorstream::service_iterators(size_type topos)
   {
     const size_type readpos = i->first;
     if (readpos > m_realpos) ignore(readpos - m_realpos);
-    const result r = fetch();
+    const result r = fetchblock();
     for ( ; i != todo_end && i->first == readpos; ++i)
       i->second->fill(r);
   }
@@ -295,7 +451,7 @@ void pqxx::icursor_iterator::refresh() const
 }
 
 
-void pqxx::icursor_iterator::fill(const result &r) const
+void pqxx::icursor_iterator::fill(const result &r)
 {
   m_here = r;
 }
