@@ -21,7 +21,12 @@
 #include <stdexcept>
 
 #include "libpq-fe.h"
-#include "libpq/libpq-fs.h"
+
+//#include "libpq/libpq-fs.h"
+/// Copied from libpq/libpq-fs.h so we don't need that header anymore
+#define INV_WRITE		0x00020000
+/// Copied from libpq/libpq-fs.h so we don't need that header anymore
+#define INV_READ		0x00040000
 
 #include "pqxx/largeobject"
 
@@ -135,19 +140,8 @@ string pqxx::largeobject::Reason(int err) const
   if (err == ENOMEM) return "Out of memory";
   if (id() == oid_none) return "No object selected";
 
-#ifndef PQXX_HAVE_STRERROR_R
-  return strerror(err);
-#else
   char buf[500];
-#ifdef PQXX_STRERROR_R_INT
-  // Single Unix Specification version: returns result code
-  if (strerror_r(err, buf, sizeof(buf)) == -1) return "Unknown error";
-  else return string(buf);
-#else
-  // GNU libc version: returns string (either in buf or elsewhere)
-  return string(strerror_r(err, buf, sizeof(buf)));
-#endif
-#endif
+  return string(strerror_wrapper(err, buf, sizeof(buf)));
 }
 
 
@@ -227,6 +221,18 @@ long pqxx::largeobjectaccess::cread(char Buf[], size_type Bytes) throw ()
 }
 
 
+pqxx::largeobjectaccess::pos_type
+pqxx::largeobjectaccess::ctell() const throw ()
+{
+  return
+#if defined(PQXX_HAVE_LO_TELL)
+        lo_tell(RawConnection(), m_fd);
+#else
+        lo_lseek(RawConnection(), m_fd, 0, SEEK_CUR);
+#endif
+}
+
+
 void pqxx::largeobjectaccess::write(const char Buf[], size_type Len)
 {
   const long Bytes = cwrite(Buf, Len);
@@ -286,6 +292,14 @@ void pqxx::largeobjectaccess::close() throw ()
 }
 
 
+pqxx::largeobjectaccess::size_type pqxx::largeobjectaccess::tell() const
+{
+  const size_type res = ctell();
+  if (res == -1) throw runtime_error(Reason(errno));
+  return res;
+}
+
+
 string pqxx::largeobjectaccess::Reason(int err) const
 {
   return (m_fd == -1) ? "No object opened" : largeobject::Reason(err);
@@ -296,4 +310,3 @@ void pqxx::largeobjectaccess::process_notice(const string &s) throw ()
 {
   m_Trans.process_notice(s);
 }
-
