@@ -140,8 +140,7 @@ string pqxx::result::StatusError() const
     break;
 
   default:
-    throw logic_error("libpqxx internal error: "
-		      "pqxx::result: Unrecognized response code " +
+    throw internal_error("pqxx::result: Unrecognized response code " +
 		      to_string(int(PQresultStatus(c_ptr()))));
   }
   return Err;
@@ -332,6 +331,14 @@ pqxx::result::const_fielditerator::operator--(int)
 }
 
 
+pqxx::result::const_iterator
+pqxx::result::const_reverse_iterator::base() const throw ()
+{
+  iterator_type tmp(*this);
+  return ++tmp;
+}
+
+
 pqxx::result::const_reverse_iterator
 pqxx::result::const_reverse_iterator::operator++(int)
 {
@@ -349,11 +356,20 @@ pqxx::result::const_reverse_iterator::operator--(int)
   return tmp;
 }
 
+
+pqxx::result::const_fielditerator
+pqxx::result::const_reverse_fielditerator::base() const throw ()
+{
+  iterator_type tmp(*this);
+  return ++tmp;
+}
+
+
 pqxx::result::const_reverse_fielditerator
 pqxx::result::const_reverse_fielditerator::operator++(int)
 {
   const_reverse_fielditerator tmp(*this);
-  iterator_type::operator--();
+  operator++();
   return tmp;
 }
 
@@ -362,7 +378,7 @@ pqxx::result::const_reverse_fielditerator
 pqxx::result::const_reverse_fielditerator::operator--(int)
 {
   const_reverse_fielditerator tmp(*this);
-  iterator_type::operator++();
+  operator--();
   return tmp;
 }
 
