@@ -16,6 +16,8 @@
  */
 #include "pqxx/compiler.h"
 
+#include <cmath>
+
 #ifdef PQXX_HAVE_LOCALE
 #include <locale>
 #endif
@@ -34,6 +36,7 @@
 
 #include "libpq-fe.h"
 
+#include "pqxx/except"
 #include "pqxx/util"
 
 using namespace PGSTD;
@@ -496,22 +499,20 @@ string pqxx::sqlesc(const char str[])
 #else
   for (size_t i=0; str[i]; ++i)
   {
-    if (isprint(str[i]))
+    // TODO: Unify these sqlesc() loop bodies!
+    // Ensure we don't pass negative integers to isprint()/isspace(), which
+    // Visual C++ chokes on.
+    const unsigned char c(str[i]);
+    if (isprint(c))
     {
-      switch (str[i])
-      {
-      case '\'':
-      case '\\':
-	result += str[i];
-      }
-      result += str[i];
+      if (c=='\\' || c=='\'') result += c;
+      result += c;
     }
     else
     {
         char s[8];
-        sprintf(s,
-	        "\\%03o",
-		static_cast<unsigned int>(static_cast<unsigned char>(str[i])));
+	// TODO: Number may be formatted according to locale!  :-(
+        sprintf(s, "\\%03o", static_cast<unsigned int>(c));
         result.append(s, 4);
     }
   }
@@ -527,22 +528,20 @@ string pqxx::sqlesc(const char str[], size_t len)
 #else
   for (size_t i=0; (i < len) && str[i]; ++i)
   {
-    if (isprint(str[i]))
+    // TODO: Unify these sqlesc() loop bodies!
+    // Ensure we don't pass negative integers to isprint()/isspace(), which
+    // Visual C++ chokes on.
+    const unsigned char c(str[i]);
+    if (isprint(c))
     {
-      switch (str[i])
-      {
-      case '\'':
-      case '\\':
-	result += str[i];
-      }
-      result += str[i];
+      if (c=='\\' || c=='\'') result += c;
+      result += c;
     }
     else
     {
         char s[8];
-        sprintf(s,
-	        "\\%03o",
-		static_cast<unsigned int>(static_cast<unsigned char>(str[i])));
+	// TODO: Number may be formatted according to locale!  :-(
+        sprintf(s, "\\%03o", static_cast<unsigned int>(c));
         result.append(s, 4);
     }
   }
@@ -554,30 +553,28 @@ string pqxx::sqlesc(const char str[], size_t len)
 
 string pqxx::sqlesc(const string &str)
 {
-  string result;
+  string res;
   const string::const_iterator str_end(str.end());
   for (string::const_iterator i = str.begin(); i != str_end; ++i)
   {
-    if (isprint(*i) || isspace(*i))
+    // TODO: Unify these sqlesc() loop bodies!
+    // Ensure we don't pass negative integers to isprint()/isspace(), which
+    // Visual C++ chokes on.
+    const unsigned char c(*i);
+    if (isprint(c) || isspace(c))
     {
-      switch (*i)
-      {
-      case '\'':
-      case '\\':
-	result += *i;
-      }
-      result += *i;
+      if (c=='\\' || c=='\'') res += c;
+      res += c;
     }
     else
     {
         char s[8];
-        sprintf(s,
-	        "\\%03o",
-		static_cast<unsigned int>(static_cast<unsigned char>(*i)));
-        result.append(s, 4);
+	// TODO: The number itself may be formatted according to locale!  :-(
+        sprintf(s, "\\%03o", static_cast<unsigned int>(c));
+        res.append(s, 4);
     }
   }
-  return result;
+  return res;
 }
 
 
@@ -627,7 +624,7 @@ void pqxx::internal::CheckUniqueRegistration(const namedclass *New,
     const namedclass *Old)
 {
   if (!New)
-    throw logic_error("libpqxx internal error: NULL pointer registered");
+    throw internal_error("NULL pointer registered");
   if (Old)
   {
     if (Old == New)
@@ -691,9 +688,65 @@ void pqxx::internal::sleep_seconds(int s)
   case ENOMEM:	// Out of memory
 	throw bad_alloc();
   default:
-    throw logic_error("libpqxx internal error in select()");
+    throw internal_error("select() failed for unknown reason");
   }
 #endif
 }
 
 
+const char *pqxx::internal::strerror_wrapper(int err, char buf[], size_t len)
+	throw ()
+{
+  if (!buf) return "No buffer provided for error message!";
+
+  const char *res = buf;
+
+#if !defined(PQXX_HAVE_STRERROR_R)
+  strncpy(buf, strerror(err), len);
+#elif defined(PQXX_HAVE_STRERROR_R_INT)
+  // Single Unix Specification version: returns result code
+  const int code = strerror_r(err, buf, len);
+  switch (code)
+  {
+  case 0: break;
+  case -1: strncpy(buf, "Unknown error", len); break;
+  // Deal with possibility that link-time strerror_r() is not the one that our
+  // configure script saw.  There's not much we can do if we really got the GNU
+  // version, which returns a char *, since in that case the pointe may have
+  // been truncated in being interpreted as an int.
+  // TODO: If this really happens, special-case sizeof(char*)==sizeof(int)
+  // TODO: Log this somewhere if possible
+  default:
+    strncpy(buf,
+	  "Unexpected result from strerror_r()!  Is this really the SUS version?",
+	  len);
+	break;
+  }
+#else
+  // GNU version; returns error string (which may be stored in buf)
+  res = strerror_r(err, buf, len);
+  switch (long(res))
+  {
+  case 0:
+    // Even if the null pointer equals zero on this system, this value can not
+	// have come out of the GNU strerror_r().  Looks like what we really got is
+	// the SUS version!
+	// TODO: Log this somewhere if possible
+	res = buf;
+	break;
+  case -1:
+	// Again, this value looks like it came out of the SUS version.  And it's an
+	// error value too, so forget about the original error and report the
+	// strerror_r() confusion so it can get fixed.
+	strncpy(buf,
+		"Unexpected result from strerror_r()!  Is this really the GNU version?",
+		len);
+	break;
+  default:
+	// This is the way it should be.
+	break;
+  }
+#endif
+  return res;
+}
+
