@@ -6,6 +6,10 @@
 #include <pqxx/transaction>
 #include <pqxx/result>
 
+// Don't try this at home: peeking inside libpqxx to see if we can test the
+// column_table() functionality
+#include <pqxx/config-internal-libpq.h>
+
 using namespace PGSTD;
 using namespace pqxx;
 
